@@ -1,20 +1,28 @@
 # -*- coding: utf-8 -*-
+import pytest
 
 from inspect import getmro
+from collections import OrderedDict
 
 from mock import Mock
 from pytest import raises
 
-from pyee import EventEmitter
+from pyee import BaseEventEmitter, EventEmitter
+
 
 class PyeeTestException(Exception):
     pass
 
-def test_emit_sync():
+
+@pytest.mark.parametrize('cls', [
+    BaseEventEmitter,
+    EventEmitter
+])
+def test_emit_sync(cls):
     """Basic synchronous emission works"""
 
     call_me = Mock()
-    ee = EventEmitter()
+    ee = cls()
 
     @ee.on('event')
     def event_handler(data, **kwargs):
@@ -27,15 +35,19 @@ def test_emit_sync():
     call_me.assert_called_once()
 
 
-def test_emit_error():
+@pytest.mark.parametrize('cls', [
+    BaseEventEmitter,
+    EventEmitter
+])
+def test_emit_error(cls):
     """Errors raise with no event handler, otherwise emit on handler"""
 
     call_me = Mock()
-    ee = EventEmitter()
+    ee = cls()
 
     test_exception = PyeeTestException('lololol')
 
-    with raises(PyeeTestException) as exc_info:
+    with raises(PyeeTestException):
         ee.emit('error', test_exception)
 
     @ee.on('error')
@@ -43,7 +55,7 @@ def test_emit_error():
         call_me()
 
     # No longer raises and error instead return True indicating handled
-    assert ee.emit('error', test_exception)
+    assert ee.emit('error', test_exception) is True
     call_me.assert_called_once()
 
 
@@ -53,7 +65,7 @@ def test_emit_return():
     """
 
     call_me = Mock()
-    ee = EventEmitter()
+    ee = BaseEventEmitter()
 
     # make sure emitting without a callback returns False
     assert not ee.emit('data')
@@ -69,7 +81,7 @@ def test_new_listener_event():
     """The 'new_listener' event fires whenever a new listerner is added."""
 
     call_me = Mock()
-    ee = EventEmitter()
+    ee = BaseEventEmitter()
 
     ee.on('new_listener', call_me)
 
@@ -78,13 +90,13 @@ def test_new_listener_event():
     def event_handler(data):
         pass
 
-    call_me.assert_called_once()
+    call_me.assert_called_once_with('event', event_handler)
 
 
 def test_listener_removal():
     """Removing listeners removes the correct listener from an event."""
 
-    ee = EventEmitter()
+    ee = BaseEventEmitter()
 
     # Some functions to pass to the EE
     def first():
@@ -105,17 +117,29 @@ def test_listener_removal():
 
     ee.on('event', fourth)
 
-    assert ee._events['event'] == [first, second, third, fourth]
+    assert ee._events['event'] == OrderedDict([
+        (first, first),
+        (second, second),
+        (third, third),
+        (fourth, fourth)
+    ])
 
     ee.remove_listener('event', second)
 
-    assert ee._events['event'] == [first, third, fourth]
+    assert ee._events['event'] == OrderedDict([
+        (first, first),
+        (third, third),
+        (fourth, fourth)
+    ])
 
     ee.remove_listener('event', first)
-    assert ee._events['event'] == [third, fourth]
+    assert ee._events['event'] == OrderedDict([
+        (third, third),
+        (fourth, fourth)
+    ])
 
     ee.remove_all_listeners('event')
-    assert ee._events['event'] == []
+    assert ee._events['event'] == OrderedDict()
 
 
 def test_listener_removal_on_emit():
@@ -124,7 +148,7 @@ def test_listener_removal_on_emit():
     """
 
     call_me = Mock()
-    ee = EventEmitter()
+    ee = BaseEventEmitter()
 
     def should_remove():
         ee.remove_listener('remove', call_me)
@@ -139,7 +163,7 @@ def test_listener_removal_on_emit():
     call_me.reset_mock()
 
     # Also test with the listeners added in the opposite order
-    ee = EventEmitter()
+    ee = BaseEventEmitter()
     ee.on('remove', call_me)
     ee.on('remove', should_remove)
 
@@ -156,44 +180,65 @@ def test_once():
     # gets removed afterwards
 
     call_me = Mock()
-    ee = EventEmitter()
+    ee = BaseEventEmitter()
 
     def once_handler(data):
         assert data == 'emitter is emitted!'
         call_me()
 
     # Tests to make sure that after event is emitted that it's gone.
-    callback_fn = ee.once('event', once_handler)
-
-    # assert ee._events['event'] == [callback_fn]
+    ee.once('event', once_handler)
 
     ee.emit('event', 'emitter is emitted!')
 
     call_me.assert_called_once()
 
-    assert ee._events['event'] == []
+    assert ee._events['event'] == OrderedDict()
+
+
+def test_once_removal():
+    """Removal of once functions works
+    """
+
+    ee = BaseEventEmitter()
+
+    def once_handler(data):
+        pass
+
+    handle = ee.once('event', once_handler)
+
+    assert handle == once_handler
+
+    ee.remove_listener('event', handle)
+
+    assert ee._events['event'] == OrderedDict()
 
 
 def test_listeners():
-    """`listeners()` gives you access to the listeners array."""
+    """`listeners()` returns a copied list of listeners."""
 
     call_me = Mock()
-    ee = EventEmitter()
+    ee = BaseEventEmitter()
 
     @ee.on('event')
     def event_handler():
         pass
 
+    @ee.once('event')
+    def once_handler():
+        pass
+
     listeners = ee.listeners('event')
 
     assert listeners[0] == event_handler
+    assert listeners[1] == once_handler
 
-    # Overwrite listener
+    # listeners is a copy, you can't mutate the innards this way
     listeners[0] = call_me
 
     ee.emit('event')
 
-    call_me.assert_called_once()
+    call_me.assert_not_called()
 
 
 def test_properties_preserved():
@@ -201,7 +246,7 @@ def test_properties_preserved():
 
     call_me = Mock()
     call_me_also = Mock()
-    ee = EventEmitter()
+    ee = BaseEventEmitter()
 
     @ee.on('always')
     def always_event_handler():
@@ -231,18 +276,19 @@ def test_properties_preserved():
 
 def test_inheritance():
     """Test that inheritance is preserved from object"""
-    assert object in getmro(EventEmitter)
+    assert object in getmro(BaseEventEmitter)
 
-    class example(EventEmitter):
+    class example(BaseEventEmitter):
         def __init__(self):
             super(example, self).__init__()
 
-    assert EventEmitter in getmro(example)
+    assert BaseEventEmitter in getmro(example)
     assert object in getmro(example)
 
+
 def test_multiple_inheritance():
     """Test that inheritance is preserved along a lengthy MRO"""
-    class example(EventEmitter):
+    class example(BaseEventEmitter):
         def __init__(self):
             super(example, self).__init__()
 
@@ -258,4 +304,4 @@ def test_multiple_inheritance():
         def __init__(self):
             super(_example2, self).__init__()
 
-    a = _example2()
+    a = _example2()  # noqa
